/* callbacks.c                                           */
/*                                                       */
/* This application demonstrates how to do outline and   */
/* opaque style dragging.  It covers both the simple     */
/* case of dragging a widget that is the child of a      */
/* window and the more complicated case (in terms of     */
/* stup)  where we are dragging something within a       */
/* parent widget that has border and margins.            */
/*                                                       */
/* This file contains:                                   */
/*                                                       */
/*   initiateDragCB()                                    */
/*   Callback type: Pt_CB_ARM                            */
/*   Widget:        base_drag_btn and pane_drag_btn      */
/*                                                       */
/*   This callback demonstrates how to initiate          */
/*   dragging through use of the PhInitDrag() function.  */
/*   The type of dragging will be is setable by the user */
/*   via a toggle button in the window.                  */
/*   The are two widgets to drag, one a child of the     */
/*   window and one a child of the pane.                 */
/*                                                       */
/*-------------------------------------------------------*/
/*                                                       */
/*   draggingCB()                                        */
/*   Callback type: Pt_CB_RAW                            */
/*   Widget:        base                                 */
/*                                                       */
/*   This callback demonstrates how handle drag events.  */
/*                                                       */
/*   Operation:                                          */
/*     For outline dragging we only want to reposition   */
/*     the widget only after dragging has been           */
/*     completed.  For opaque dragging we want to        */
/*     reposition the widget as dragging is being done.  */
/*                                                       */
/*-------------------------------------------------------*/
/*                                                       */
/*   setDraggingTypeCB()                                 */
/*   Callback type: Pt_CB_ACTIVATE                       */
/*   Widget:        do_outline_btn                       */
/*                                                       */
/*   This simply allows the user to toggle outline vs.   */
/*   opaque dragging.                                    */
/*                                                       */
/*                            AppBuilder Photon Code Lib */
/*                                         Version 1.11B */

/* Standard headers */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

/* Toolkit headers */
#include <Ph.h>
#include <Pt.h>
#include <Ap.h>

/* Local headers */
#include "abimport.h"
#include "proto.h"

PtWidget_t  *dragging_widget;
int         do_outline_dragging = 1;

/*----------------------------------------------------------------------
 *
 * initiateDraggingCB
 *
 *---------------------------------------------------------------------*/
int
initiateDraggingCB (PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo)
{
    PhArea_t    da_area;
    PhRect_t    da_canvas;      // dragging area, boundary for dragging
    PhRect_t    dw_rect;        // rectangle to drag (dragging widget rect)
    PtWidget_t  *dragging_area;
    unsigned    drag_flags;
    
    dragging_widget = widget;
    dragging_area = PtWidgetParent( widget );
    
    /*
     * Get the bounding area for the dragging, put in da_canvas.
     * Get the rectangular area of the widget being dragged, put in dw_rect.
     *
     * Keep in mind that we want to restrict dragging to within any
     * border and/or margins of the parent.  Our window widget doesn't have
     * any but the pane widget has both.
     */
    
	PtBasicWidgetCanvas( dragging_area, &da_canvas );
    PtWidgetArea( dragging_area, &da_area );
	PtDeTranslateRect( &da_canvas, &da_area.pos );
	// da_canvas is now relative to dragging_area's region.
    // It is the canvas of the dragging area (inside of border and margins).

    PtWidgetExtent( dragging_widget, &dw_rect );
    PtTranslateRect( &dw_rect, &da_canvas.ul );
    // dw_rect is now relative to the dragging_area's region.
    
    if ( do_outline_dragging )
        drag_flags = Ph_TRACK_DRAG;
    else
        drag_flags = Ph_TRACK_DRAG | Ph_DRAG_TRACK;

    PhInitDrag( PtWidgetRid (dragging_area), drag_flags,
                &dw_rect, &da_canvas, 
                cbinfo->event->input_group,
                NULL, NULL, NULL );
                
	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	return( Pt_CONTINUE );
}

/*----------------------------------------------------------------------
 *
 * draggingCB
 *
 *---------------------------------------------------------------------*/
int
draggingCB (PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo)
{
    PhArea_t        da_area;
    PhDragEvent_t   *evdata;
    PhRect_t        da_canvas;  // dragging area (boundary for dragging)
    PhRect_t        dw_rect;    // rectangle to drag (dragging widget rect)
    PtArg_t         arg[1];
    PtWidget_t      *dragging_area;
    
    dragging_area = widget;

    if ( do_outline_dragging ) {
        if (cbinfo->event->subtype != Ph_EV_DRAG_COMPLETE)
            return (Pt_CONTINUE); // we only care when user completes dragging
    } else { // otherwise doing opaque dragging
        if (cbinfo->event->subtype != Ph_EV_DRAG_MOVE)
            return (Pt_CONTINUE); // we only care while the user is dragging
    }
        
    evdata = PhGetData( cbinfo->event );

    // evdata->rect contains absolute coordinates of the dragging rectangle
    dw_rect = evdata->rect;
    
	PtTranslateRect( &dw_rect, &cbinfo->event->translation );
    // dw_rect is now relative to the dragging_area's region

	PtBasicWidgetCanvas( dragging_area, &da_canvas );
    PtWidgetArea( dragging_area, &da_area );
	PtDeTranslateRect( &da_canvas, &da_area.pos );
	// da_canvas is now relative to dragging_area's region.
    // It is the canvas of the dragging area (inside of border and margins).

    PtDeTranslateRect( &dw_rect, &da_canvas.ul );
    // dw_rect is now relative to the dragging_area's canvas

    /*
     * reposition the widget to it's new location.
     */
    
	PtSetArg( &arg[0], Pt_ARG_POS, &dw_rect.ul, 0 );
    PtSetResources( dragging_widget, 1, arg );
    
	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	return( Pt_CONTINUE );
}

/*----------------------------------------------------------------------
 *
 * setDraggingTypeCB
 *
 *---------------------------------------------------------------------*/
int
setDraggingTypeCB( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )
{
    do_outline_dragging = !do_outline_dragging;

	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	return( Pt_CONTINUE );
}

